@@ -26,10 +26,9 @@
 #include <apr_general.h>
 #include <apr_lib.h>
 
-#include <svn_pools.h>
-#include <svn_config.h>
+#include "svn_pools.h"
 #include "svn_wc.h"
-#include <svn_path.h>
+#include "svn_path.h"
 #include <apr_file_info.h>
 #include "svn_private_config.h"
 #ifdef WIN32
@@ -38,7 +37,7 @@
    that should be fine for now, but a better solution must be found in
    combination with issue #850. */
 extern "C" {
-#include "arch/win32/apr_arch_utf8.h"
+#include <arch/win32/apr_arch_utf8.h>
 };
 #endif
 
@@ -193,8 +192,8 @@ bool JNIUtil::JNIGlobalInit(JNIEnv *env)
         inwords = lstrlenW (ucs2_path);
         outbytes = outlength = 3 * (inwords + 1);
         utf8_path = (char *)apr_palloc (pool, outlength);
-        apr_err = apr_conv_ucs2_to_utf8 (ucs2_path, &inwords,
-                                         utf8_path, &outbytes);
+        apr_err = apr_conv_ucs2_to_utf8 ((const apr_wchar_t *) ucs2_path,
+                                         &inwords, utf8_path, &outbytes);
         if (!apr_err && (inwords > 0 || outbytes == 0))
           apr_err = APR_INCOMPLETE;
         if (apr_err)
@@ -242,16 +241,6 @@ bool JNIUtil::JNIGlobalInit(JNIEnv *env)
     }
 #endif
 
-    // we use the default directory for config files
-    // this can be changed later
-    svn_error_t *err = svn_config_ensure (NULL, g_pool); 
-    if (err)
-    {
-        svn_pool_destroy (g_pool);
-        handleSVNError(err);
-        return false;
-    }
-
     // build all mutexes
     g_finalizedObjectsMutex = new JNIMutex(g_pool);
     if(isExceptionThrown())
@@ -303,20 +292,16 @@ JNIMutex *JNIUtil::getGlobalPoolMutex()
 {
     return g_globalPoolMutext;
 }
-/**
- * throw an error
- * @param message the message text of the error
- */
-void JNIUtil::throwError(const char *message)
+void JNIUtil::raiseThrowable(const char *name, const char *message)
 {
-    if(getLogLevel() >= errorLog)
+    if (getLogLevel() >= errorLog)
     {
         JNICriticalSection cs(*g_logMutex);
-        g_logStream << "Error thrown <" << message << ">" << std::endl;
+        g_logStream << "Throwable raised <" << message << ">" << std::endl;
     }
     JNIEnv *env = getEnv();
-    jclass clazz = env->FindClass(JAVA_PACKAGE"/JNIError");
-    if(isJavaExceptionThrown())
+    jclass clazz = env->FindClass(name);
+    if (isJavaExceptionThrown())
     {
         return;
     }
@@ -405,28 +390,21 @@ void JNIUtil::handleSVNError(svn_error_t *err)
     }
     env->Throw(static_cast<jthrowable>(error));
 }
-/**
- * put the object on the finalized object list
- * it will be deleted by another thread during the next request
- * @param cl    the C++ peer of the finalized object
- */
-void JNIUtil::putFinalizedClient(SVNBase *cl)
+
+void JNIUtil::putFinalizedClient(SVNBase *object)
 {
-    // log this, because the object should have disposed
-    if(getLogLevel() >= errorLog)
-    {
-        JNICriticalSection cs(*g_logMutex);
-        g_logStream << "a client object was not disposed" << std::endl;
-    }
-    JNICriticalSection cs(*g_finalizedObjectsMutex);
-    if(isExceptionThrown())
-    {
-        return;
-    }
-
-    g_finalizedObjects.push_back(cl);
-
+    enqueueForDeletion(object);
 }
+
+void JNIUtil::enqueueForDeletion(SVNBase *object)
+{
+    JNICriticalSection cs(*g_finalizedObjectsMutex);
+    if (!isExceptionThrown())
+    {
+        g_finalizedObjects.push_back(object);
+    }
+}
+
 /**
  * Handle an apr error (those are not expected) by throwing an error
  * @param error the apr error number
@@ -736,10 +714,9 @@ void JNIUtil::assembleErrorMessage(svn_error_t *err, int depth,
  */
 void JNIUtil::throwNullPointerException(const char *message)
 {
-    if(getLogLevel() >= errorLog)
+    if (getLogLevel() >= errorLog)
     {
-        JNICriticalSection cs(*g_logMutex);
-        g_logStream << "NullPointerException thrown" << std::endl;
+        logMessage("NullPointerException thrown");
     }
     JNIEnv *env = getEnv();
     jclass clazz = env->FindClass("java/lang/NullPointerException");
