@@ -59,18 +59,56 @@ public:
     static bool isJavaExceptionThrown();
     static JNIEnv * getEnv();
     static void setEnv(JNIEnv *);
+
+    /**
+     * @return Whether any Throwable has been raised.
+     */
     static bool isExceptionThrown();
+
     static void handleAPRError(int error, const char *op);
+
+    /**
+     * Put @a object in the list of finalized objects queued up to be
+     * deleted (by another thread) during the next operation.
+     *
+     * @param object The C++ peer of the finalized (Java) object.
+     * @since 1.4.0
+     */
+    static void enqueueForDeletion(SVNBase *object);
+
+    /**
+     * @deprecated Use the more appropriately named
+     * enqueueForDeletion() instead.
+     */
     static void putFinalizedClient(SVNBase *cl);
+
     static void handleSVNError(svn_error_t *err);
     static jstring makeSVNErrorMessage(svn_error_t *err);
-    static void throwError(const char *message);
+
+    /**
+     * Create and throw a java.lang.Throwable instance.
+     *
+     * @param name The class name (in path form, with slashes in lieu
+     * of dots) of the Throwable to create and raise.
+     * @param message The message text of the Throwable.
+     */
+    static void raiseThrowable(const char *name, const char *message);
+
+    /**
+     * Creates and throws a JNIError.
+     *
+     * @param message The message text of the JNIError.
+     */
+    static void throwError(const char *message)
+    { raiseThrowable(JAVA_PACKAGE"/JNIError", message); }
+
     static apr_pool_t * getPool();
 	static bool JNIGlobalInit(JNIEnv *env);
     static bool JNIInit(JNIEnv *env);
     static JNIMutex *getGlobalPoolMutex();
     enum { formatBufferSize = 2048 };
     enum { noLog, errorLog, exceptionLog, entryLog } LogLevel;
+
 private:
     static void assembleErrorMessage(svn_error_t *err, int depth,
                                          apr_status_t parent_apr_err,
