@@ -3,7 +3,7 @@ package org.tigris.subversion.javahl;
 /**
  * @copyright
  * ====================================================================
- * Copyright (c) 2003-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2003-2005 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -17,6 +17,9 @@ package org.tigris.subversion.javahl;
  * ====================================================================
  * @endcopyright
  */
+
+import java.io.OutputStream;
+
 /**
  * This is the main interface class. All subversion commandline client svn &
  * svnversion operation are implemented in this class. This class is not
@@ -25,44 +28,11 @@ package org.tigris.subversion.javahl;
 public class SVNClient implements SVNClientInterface
 {
     /**
-     * load the needed native library
+     * Load the required native library.
      */
     static
     {
-        /*
-         * see if the user has specified the fully qualified path to the native
-         * library
-         */
-        try
-        {
-            String specifiedLibraryName =
-                    System.getProperty("subversion.native.library");
-            if(specifiedLibraryName != null)
-                System.load(specifiedLibraryName);
-        }
-        catch(UnsatisfiedLinkError ex)
-        {
-            // ignore that error to try again
-        }
-        /*
-         * first try to load the library by the new name.
-         * if that fails, try to load the library by the old name.
-         */
-        try
-        {
-            System.loadLibrary("svnjavahl-1");
-        }
-        catch(UnsatisfiedLinkError ex)
-        {
-            try
-            {
-                System.loadLibrary("libsvnjavahl-1");
-            }
-            catch (UnsatisfiedLinkError e)
-            {
-                System.loadLibrary("svnjavahl");
-            }
-        }
+        NativeResources.loadNativeLibrary();
     }
 
     /**
@@ -71,6 +41,20 @@ public class SVNClient implements SVNClientInterface
     public SVNClient()
     {
         cppAddr = ctNative();
+
+        // Ensure that Subversion's config file area and templates exist.
+        try
+        {
+            // Passing an empty string instead of null (which would be
+            // more appropriate) prevents earlier versions of the
+            // native library (mismatched from the Java bytecode
+            // version) from crashing.
+            setConfigDirectory("");
+        }
+        catch (ClientException suppressed)
+        {
+            // Not an exception-worthy problem, continue on.
+        }
     }
     /**
      * Build the native peer
@@ -90,6 +74,30 @@ public class SVNClient implements SVNClientInterface
      * of this member
      */
     protected long cppAddr;
+    /**
+     * @return Version information about the underlying native libraries.
+     */
+    public Version getVersion()
+    {
+        return NativeResources.version;
+    }
+    /**
+     * @return The name of the working copy's administrative
+     * directory, which is usually <code>.svn</code>.
+     * @see <a
+     * href="http://svn.collab.net/repos/svn/trunk/notes/asp-dot-net-hack.txt">Instructions</a>
+     * on changing this as a work-around for the behavior of ASP.Net
+     * on Windows.
+     * @since 1.3
+     */
+    public native String getAdminDirectoryName();
+    /**
+     * @param name The name of the directory to compare.
+     * @return Whether <code>name</code> is that of a working copy
+     * administrative directory.
+     * @since 1.3
+     */
+    public native boolean isAdminDirectory(String name);
     /**
       * Returns the last destination path submitted.
       * @deprecated
@@ -100,13 +108,13 @@ public class SVNClient implements SVNClientInterface
      * List a directory or file of the working copy.
      *
      * @param path      Path to explore.
-     * @param descend   Recurse into subdirectories if existant.
+     * @param descend   Recurse into subdirectories if they exist.
      * @param onServer  Request status information from server.
      * @param getAll    get status for uninteristing files (unchanged).
      * @return Array of Status entries.
      */
-    public Status[]status(String path, boolean descend, boolean onServer,
-                          boolean getAll) throws ClientException
+    public Status[] status(String path, boolean descend, boolean onServer,
+                           boolean getAll) throws ClientException
     {
         return status(path, descend, onServer, getAll, false);
     }
@@ -114,7 +122,7 @@ public class SVNClient implements SVNClientInterface
      * List a directory or file of the working copy.
      *
      * @param path      Path to explore.
-     * @param descend   Recurse into subdirectories if existant.
+     * @param descend   Recurse into subdirectories they exist.
      * @param onServer  Request status information from server.
      * @param getAll    get status for uninteristing files (unchanged).
      * @param noIgnore  get status for normaly ignored files and directories.
@@ -131,7 +139,7 @@ public class SVNClient implements SVNClientInterface
      * List a directory or file of the working copy.
      *
      * @param path            Path to explore.
-     * @param descend         Recurse into subdirectories if existant.
+     * @param descend         Recurse into subdirectories if they exist.
      * @param onServer        Request status information from server.
      * @param getAll          get status for uninteristing files (unchanged).
      * @param noIgnore        get status for normaly ignored files and
@@ -152,7 +160,7 @@ public class SVNClient implements SVNClientInterface
      * @param recurse   recurse into subdirectories
      * @return  Array of DirEntry objects.
      */
-    public DirEntry[]list(String url, Revision revision, boolean recurse)
+    public DirEntry[] list(String url, Revision revision, boolean recurse)
             throws ClientException
     {
         return list(url, revision, revision, recurse);
@@ -182,8 +190,8 @@ public class SVNClient implements SVNClientInterface
     public native Status singleStatus(String path, boolean onServer)
             throws ClientException;
     /**
-     * Sets the username used for authentification.
-     * @param username  the username
+     * Sets the user name used for authentification.
+     * @param username The user name.
      */
     public native void username(String username);
     /**
@@ -192,7 +200,12 @@ public class SVNClient implements SVNClientInterface
      */
     public native void password(String password);
     /**
-     * Register callback interface to supply username and password on demand
+     * Register callback interface to supply user name and password on
+     * demand.  This callback can also be used to provide the
+     * equivalent of the <code>--no-auth-cache</code> and
+     * <code>--non-interactive</code> arguments accepted by the
+     * command-line client.
+     *
      * @param prompt the callback interface
      */
     public native void setPrompt(PromptUserPassword prompt);
@@ -921,6 +934,24 @@ public class SVNClient implements SVNClientInterface
                                      Revision pegRevision)
             throws ClientException;
 
+    /**
+     * Write the file's content to the specified output stream.  If
+     * you need an InputStream, use a
+     * PipedInputStream/PipedOutputStream combination.
+     *
+     * @param path        the path of the file
+     * @param revision    the revision to retrieve
+     * @param pegRevision the revision at which to interpret the path
+     * @param stream      the stream to write the file's content to
+     * @throws ClientException
+     * @see <a href="http://java.sun.com/j2se/1.4.2/docs/api/java/io/PipedOutputStream.html">PipedOutputStream</a>
+     * @see <a href="http://java.sun.com/j2se/1.4.2/docs/api/java/io/PipedInputStream.html">PipedInputStream</a>
+     */
+    public native void streamFileContent(String path, Revision revision, 
+                            Revision pegRevision, int bufferSize, 
+                            OutputStream stream) 
+            throws ClientException;    
+
     /**
      * Rewrite the url's in the working copy
      * @param from      old url
@@ -1106,4 +1137,9 @@ public class SVNClient implements SVNClientInterface
     public native Info2[] info2(String pathOrUrl, Revision revision,
                                 Revision pegRevision, boolean recurse)
             throws ClientException;
+    /**
+     * Internal method to initialize the native layer. Only to be called by
+     * NativeResources.loadNativeLibrary
+     */
+    static native void initNative();
 }
