@@ -30,6 +30,7 @@
 #include <apr_portable.h>
 #include <apr_thread_proc.h>
 
+#include "svn_client.h"
 #include "svn_string.h"
 #include "svn_opt.h"
 #include "svn_delta.h"
@@ -550,6 +551,7 @@ DECLARE_SWIG_CONSTRUCTOR(wc_status, svn_wc_dup_status)
 DECLARE_SWIG_CONSTRUCTOR(lock, svn_lock_dup)
 DECLARE_SWIG_CONSTRUCTOR(auth_ssl_server_cert_info,
     svn_auth_ssl_server_cert_info_dup)
+DECLARE_SWIG_CONSTRUCTOR(info, svn_info_dup)
 
 static PyObject *convert_log_changed_path(void *value, void *ctx,
                                           PyObject *py_pool)
@@ -602,8 +604,6 @@ apr_hash_t *svn_swig_py_stringhash_from_dict(PyObject *dict,
       PyObject *value = PyDict_GetItem(dict, key);
       const char *propname = make_string_from_ob(key, pool);
       const char *propval = make_string_from_ob(value, pool);
-      Py_DECREF(key);
-      Py_DECREF(value);
       if (! (propname && propval))
         {
           PyErr_SetString(PyExc_TypeError, 
@@ -642,8 +642,6 @@ apr_hash_t *svn_swig_py_prophash_from_dict(PyObject *dict,
       PyObject *value = PyDict_GetItem(dict, key);
       const char *propname = make_string_from_ob(key, pool);
       svn_string_t *propval = make_svn_string_from_ob(value, pool);
-      Py_DECREF(key);
-      Py_DECREF(value);
       if (! (propname && propval))
         {
           PyErr_SetString(PyExc_TypeError, 
@@ -835,11 +833,11 @@ static item_baton *make_baton(apr_pool_t *pool,
 {
   item_baton *newb = apr_palloc(pool, sizeof(*newb));
 
-  /* one more reference to the editor. */
-  Py_INCREF(editor);
-
-  /* note: we take the caller's reference to 'baton' */
-
+  /* Note: We steal the caller's reference to 'baton'. Also, to avoid
+     memory leaks, we borrow the caller's reference to 'editor'. In this
+     case, borrowing the reference to 'editor' is safe because the contents
+     of an item_baton struct are only used by functino calls which operate on
+     the editor itself. */
   newb->editor = editor;
   newb->baton = baton;
 
@@ -873,7 +871,6 @@ static svn_error_t *close_baton(void *baton,
   /* We're now done with the baton. Since there isn't really a free, all
      we need to do is note that its objects are no longer referenced by
      the baton.  */
-  Py_DECREF(ib->editor);
   Py_XDECREF(ib->baton);
 
 #ifdef SVN_DEBUG
@@ -1281,7 +1278,6 @@ static svn_error_t *close_file(void *file_baton,
   /* We're now done with the baton. Since there isn't really a free, all
      we need to do is note that its objects are no longer referenced by
      the baton.  */
-  Py_DECREF(ib->editor);
   Py_XDECREF(ib->baton);
 
 #ifdef SVN_DEBUG
@@ -1312,7 +1308,7 @@ void svn_swig_py_make_editor(const svn_delta_editor_t **editor,
                              PyObject *py_editor,
                              apr_pool_t *pool)
 {
-  svn_delta_editor_t *thunk_editor = svn_delta_default_editor (pool);
+  svn_delta_editor_t *thunk_editor = svn_delta_default_editor(pool);
   
   thunk_editor->set_target_revision = set_target_revision;
   thunk_editor->open_root = open_root;
@@ -1338,8 +1334,8 @@ void svn_swig_py_make_editor(const svn_delta_editor_t **editor,
 /*** Other Wrappers for SVN Functions ***/
 
 
-apr_file_t *svn_swig_py_make_file (PyObject *py_file,
-                                   apr_pool_t *pool)
+apr_file_t *svn_swig_py_make_file(PyObject *py_file,
+                                  apr_pool_t *pool)
 {
   apr_file_t *apr_file = NULL;
   apr_status_t apr_err;
@@ -1347,23 +1343,23 @@ apr_file_t *svn_swig_py_make_file (PyObject *py_file,
   if (py_file == NULL || py_file == Py_None)
     return NULL;
 
-  if (PyString_Check (py_file))
+  if (PyString_Check(py_file))
     {
       /* input is a path -- just open an apr_file_t */
-      char* fname = PyString_AS_STRING (py_file);
-      apr_err = apr_file_open (&apr_file, fname,
+      char* fname = PyString_AS_STRING(py_file);
+      apr_err = apr_file_open(&apr_file, fname,
                               APR_CREATE | APR_READ | APR_WRITE,
                               APR_OS_DEFAULT, pool);
       if (apr_err)
         {
           char buf[256];
-          apr_strerror (apr_err, buf, sizeof(buf));
-          PyErr_Format (PyExc_IOError, "apr_file_open failed: %s: '%s'",
-                        buf, fname);
+          apr_strerror(apr_err, buf, sizeof(buf));
+          PyErr_Format(PyExc_IOError, "apr_file_open failed: %s: '%s'",
+                       buf, fname);
           return NULL;
         }
     }
-  else if (PyFile_Check (py_file))
+  else if (PyFile_Check(py_file))
     {
       FILE *file;
       apr_os_file_t osfile;
@@ -1375,12 +1371,12 @@ apr_file_t *svn_swig_py_make_file (PyObject *py_file,
 #else
       osfile = (apr_os_file_t)fileno(file);
 #endif
-      apr_err = apr_os_file_put (&apr_file, &osfile, O_CREAT | O_WRONLY, pool);
+      apr_err = apr_os_file_put(&apr_file, &osfile, O_CREAT | O_WRONLY, pool);
       if (apr_err)
         {
           char buf[256];
-          apr_strerror (apr_err, buf, sizeof(buf));
-          PyErr_Format (PyExc_IOError, "apr_os_file_put failed: %s", buf);
+          apr_strerror(apr_err, buf, sizeof(buf));
+          PyErr_Format(PyExc_IOError, "apr_os_file_put failed: %s", buf);
           return NULL;
         }
     }
@@ -1389,7 +1385,7 @@ apr_file_t *svn_swig_py_make_file (PyObject *py_file,
 
 
 static svn_error_t *
-read_handler_pyio (void *baton, char *buffer, apr_size_t *len)
+read_handler_pyio(void *baton, char *buffer, apr_size_t *len)
 {
   PyObject *result;
   PyObject *py_io = baton;
@@ -1427,7 +1423,7 @@ read_handler_pyio (void *baton, char *buffer, apr_size_t *len)
 }
 
 static svn_error_t *
-write_handler_pyio (void *baton, const char *data, apr_size_t *len)
+write_handler_pyio(void *baton, const char *data, apr_size_t *len)
 {
   PyObject *result;
   PyObject *py_io = baton;
@@ -1449,7 +1445,7 @@ write_handler_pyio (void *baton, const char *data, apr_size_t *len)
 }
 
 svn_stream_t *
-svn_swig_py_make_stream (PyObject *py_io, apr_pool_t *pool)
+svn_swig_py_make_stream(PyObject *py_io, apr_pool_t *pool)
 {
   svn_stream_t *stream;
 
@@ -1458,9 +1454,9 @@ svn_swig_py_make_stream (PyObject *py_io, apr_pool_t *pool)
    * bindings, and we will be finished with the py_io object before we return
    * to python. I.e. DO NOT STORE AWAY THE RESULTING svn_stream_t * for use
    * over multiple calls into the bindings. */
-  stream = svn_stream_create (py_io, pool);
-  svn_stream_set_read (stream, read_handler_pyio);
-  svn_stream_set_write (stream, write_handler_pyio);
+  stream = svn_stream_create(py_io, pool);
+  svn_stream_set_read(stream, read_handler_pyio);
+  svn_stream_set_write(stream, write_handler_pyio);
 
   return stream;
 }
@@ -1577,9 +1573,9 @@ svn_error_t *svn_swig_py_cancel_func(void *cancel_baton)
   return err;
 }
 
-svn_error_t *svn_swig_py_fs_get_locks_func (void *baton, 
-                                            svn_lock_t *lock,
-                                            apr_pool_t *pool)
+svn_error_t *svn_swig_py_fs_get_locks_func(void *baton, 
+                                           svn_lock_t *lock,
+                                           apr_pool_t *pool)
 {
   PyObject *function = baton;
   PyObject *result;
@@ -1816,6 +1812,38 @@ finished:
   return err;
 }
 
+svn_error_t *svn_swig_py_info_receiver_func(void *baton,
+                                            const char *path,
+                                            const svn_info_t *info,
+                                            apr_pool_t *pool)
+{
+  PyObject *receiver = baton;
+  PyObject *result;
+  svn_error_t *err = SVN_NO_ERROR;
+ 
+  if ((receiver == NULL) || (receiver == Py_None))
+    return SVN_NO_ERROR;
+
+  svn_swig_py_acquire_py_lock();
+
+  if ((result = PyObject_CallFunction(receiver, 
+                                      (char *)"sO&O&", 
+                                      path, make_ob_info, info,
+                                      make_ob_pool, pool)) == NULL)
+    {
+      err = callback_exception_error();
+    }
+  else
+    {
+      if (result != Py_None)
+        err = callback_bad_return_error("Not None");
+      Py_DECREF(result);
+    }
+
+  svn_swig_py_release_py_lock();
+
+  return err;
+}
 
 svn_error_t *svn_swig_py_client_blame_receiver_func(void *baton,
                                                     apr_int64_t line_no,
@@ -1855,12 +1883,12 @@ svn_error_t *svn_swig_py_client_blame_receiver_func(void *baton,
 
 
 svn_error_t *
-svn_swig_py_auth_simple_prompt_func (svn_auth_cred_simple_t **cred,
-                                     void *baton,
-                                     const char *realm,
-                                     const char *username,
-                                     svn_boolean_t may_save,
-                                     apr_pool_t *pool)
+svn_swig_py_auth_simple_prompt_func(svn_auth_cred_simple_t **cred,
+                                    void *baton,
+                                    const char *realm,
+                                    const char *username,
+                                    svn_boolean_t may_save,
+                                    apr_pool_t *pool)
 {
   PyObject *function = baton;
   PyObject *result;
@@ -1891,7 +1919,7 @@ svn_swig_py_auth_simple_prompt_func (svn_auth_cred_simple_t **cred,
             }
           else
             {
-              creds = apr_pcalloc(pool, sizeof (*creds));
+              creds = apr_pcalloc(pool, sizeof(*creds));
               creds->username = tmp_creds->username ? \
                 apr_pstrdup(pool, tmp_creds->username) : NULL;
               creds->password = tmp_creds->password ? \
@@ -1907,11 +1935,11 @@ svn_swig_py_auth_simple_prompt_func (svn_auth_cred_simple_t **cred,
 }
 
 svn_error_t *
-svn_swig_py_auth_username_prompt_func (svn_auth_cred_username_t **cred,
-                                       void *baton,
-                                       const char *realm,
-                                       svn_boolean_t may_save,
-                                       apr_pool_t *pool)
+svn_swig_py_auth_username_prompt_func(svn_auth_cred_username_t **cred,
+                                      void *baton,
+                                      const char *realm,
+                                      svn_boolean_t may_save,
+                                      apr_pool_t *pool)
 {
   PyObject *function = baton;
   PyObject *result;
@@ -1942,7 +1970,7 @@ svn_swig_py_auth_username_prompt_func (svn_auth_cred_username_t **cred,
             }
           else
             {
-              creds = apr_pcalloc(pool, sizeof (*creds));
+              creds = apr_pcalloc(pool, sizeof(*creds));
               creds->username = tmp_creds->username ? \
                 apr_pstrdup(pool, tmp_creds->username) : NULL;
               creds->may_save = tmp_creds->may_save;
@@ -1996,7 +2024,7 @@ svn_swig_py_auth_ssl_server_trust_prompt_func(
             }
           else
             {
-              creds = apr_pcalloc(pool, sizeof (*creds));
+              creds = apr_pcalloc(pool, sizeof(*creds));
               *creds = *tmp_creds;
             }
         }
@@ -2046,7 +2074,7 @@ svn_swig_py_auth_ssl_client_cert_prompt_func(
             }
           else
             {
-              creds = apr_pcalloc(pool, sizeof (*creds));
+              creds = apr_pcalloc(pool, sizeof(*creds));
               creds->cert_file = tmp_creds->cert_file ? \
                 apr_pstrdup(pool, tmp_creds->cert_file) : NULL;
               creds->may_save = tmp_creds->may_save;
@@ -2098,7 +2126,7 @@ svn_swig_py_auth_ssl_client_cert_pw_prompt_func(
             }
           else
             {
-              creds = apr_pcalloc(pool, sizeof (*creds));
+              creds = apr_pcalloc(pool, sizeof(*creds));
               creds->password = tmp_creds->password ? \
                 apr_pstrdup(pool, tmp_creds->password) : NULL;
               creds->may_save = tmp_creds->may_save;
