@@ -22,12 +22,9 @@
 
 /*** Includes. ***/
 
+#include "svn_time.h"
 #include "svn_wc.h"
 #include "svn_client.h"
-#include "svn_string.h"
-#include "svn_pools.h"
-#include "svn_error.h"
-#include "svn_path.h"
 #include "svn_config.h"
 #include "client.h"
 
@@ -37,26 +34,22 @@
 /*** Code. ***/
 
 svn_error_t *
-svn_client_cleanup (const char *dir,
-                    svn_client_ctx_t *ctx,
-                    apr_pool_t *pool)
+svn_client_cleanup(const char *dir,
+                   svn_client_ctx_t *ctx,
+                   apr_pool_t *pool)
 {
-  svn_node_kind_t kind;
   const char *diff3_cmd;
+  svn_error_t *err;
   svn_config_t *cfg = ctx->config
-    ? apr_hash_get (ctx->config, SVN_CONFIG_CATEGORY_CONFIG,  
-                    APR_HASH_KEY_STRING)
+    ? apr_hash_get(ctx->config, SVN_CONFIG_CATEGORY_CONFIG,  
+                   APR_HASH_KEY_STRING)
     : NULL;
 
-  SVN_ERR (svn_io_check_path (dir, &kind, pool));
-  if (kind != svn_node_dir)
-    return svn_error_createf (SVN_ERR_WC_NOT_DIRECTORY, NULL,
-                              _("Cannot cleanup '%s': not a directory"),
-                              dir);
+  svn_config_get(cfg, &diff3_cmd, SVN_CONFIG_SECTION_HELPERS,
+                 SVN_CONFIG_OPTION_DIFF3_CMD, NULL);
 
-  svn_config_get (cfg, &diff3_cmd, SVN_CONFIG_SECTION_HELPERS,
-                  SVN_CONFIG_OPTION_DIFF3_CMD, NULL);
-
-  return svn_wc_cleanup (dir, NULL, diff3_cmd,
-                         ctx->cancel_func, ctx->cancel_baton, pool);
+  err = svn_wc_cleanup2(dir, diff3_cmd, ctx->cancel_func, ctx->cancel_baton,
+                        pool);
+  svn_sleep_for_timestamps();
+  return err;
 }
