@@ -2,7 +2,7 @@
  * client.h :  shared stuff internal to the client library.
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -61,19 +61,19 @@ extern "C" {
  * Use POOL for any temporary allocation.
  */
 svn_error_t *
-svn_client__get_revision_number (svn_revnum_t *revnum,
-                                 svn_ra_session_t *ra_session,
-                                 const svn_opt_revision_t *revision,
-                                 const char *path,
-                                 apr_pool_t *pool);
+svn_client__get_revision_number(svn_revnum_t *revnum,
+                                svn_ra_session_t *ra_session,
+                                const svn_opt_revision_t *revision,
+                                const char *path,
+                                apr_pool_t *pool);
 
 /* Return true if REVISION1 and REVISION2 would result in the same
    revision number if interpreted in the context of the same working
    copy and path and repository, or if both are of kind
    svn_opt_revision_unspecified.  Otherwise, return false. */
 svn_boolean_t
-svn_client__compare_revisions (svn_opt_revision_t *revision1,
-                               svn_opt_revision_t *revision2);
+svn_client__compare_revisions(svn_opt_revision_t *revision1,
+                              svn_opt_revision_t *revision2);
 
 
 /* Return true if the revision number for REVISION can be determined
@@ -84,7 +84,7 @@ svn_client__compare_revisions (svn_opt_revision_t *revision1,
    day there are, this will return true for those kinds.
  */ 
 svn_boolean_t 
-svn_client__revision_is_local (const svn_opt_revision_t *revision);
+svn_client__revision_is_local(const svn_opt_revision_t *revision);
 
 
 /* Given the CHANGED_PATHS and REVISION from an instance of a
@@ -100,14 +100,14 @@ svn_client__revision_is_local (const svn_opt_revision_t *revision);
    This is useful for tracking the various changes in location a
    particular resource has undergone when performing an RA->get_logs()
    operation on that resource.  */
-svn_error_t *svn_client__prev_log_path (const char **prev_path_p,
-                                        char *action_p,
-                                        svn_revnum_t *copyfrom_rev_p,
-                                        apr_hash_t *changed_paths,
-                                        const char *path,
-                                        svn_node_kind_t kind,
-                                        svn_revnum_t revision,
-                                        apr_pool_t *pool);
+svn_error_t *svn_client__prev_log_path(const char **prev_path_p,
+                                       char *action_p,
+                                       svn_revnum_t *copyfrom_rev_p,
+                                       apr_hash_t *changed_paths,
+                                       const char *path,
+                                       svn_node_kind_t kind,
+                                       svn_revnum_t revision,
+                                       apr_pool_t *pool);
 
 
 /* Set *START_URL and *START_REVISION (and maybe *END_URL
@@ -118,10 +118,13 @@ svn_error_t *svn_client__prev_log_path (const char **prev_path_p,
    specify the point(s) of interest (typically the revisions referred
    to as the "operative range" for a given operation) along that history.
 
-   eND may be of kind svn_opt_revision_unspecified (in which case
+   END may be of kind svn_opt_revision_unspecified (in which case
    END_URL and END_REVISION are not touched by the function);
    START and REVISION may not.
 
+   RA_SESSION should be an open RA session pointing at the URL of PATH,
+   or NULL, in which case this function will open its own temporary session.
+
    A NOTE ABOUT FUTURE REPORTING:
 
    If either START or END are greater than REVISION, then do a
@@ -137,16 +140,17 @@ svn_error_t *svn_client__prev_log_path (const char **prev_path_p,
 
    Use POOL for all allocations.  */
 svn_error_t *
-svn_client__repos_locations (const char **start_url,
-                             svn_opt_revision_t **start_revision,
-                             const char **end_url,
-                             svn_opt_revision_t **end_revision,
-                             const char *path,
-                             const svn_opt_revision_t *revision,
-                             const svn_opt_revision_t *start,
-                             const svn_opt_revision_t *end,
-                             svn_client_ctx_t *ctx,
-                             apr_pool_t *pool);
+svn_client__repos_locations(const char **start_url,
+                            svn_opt_revision_t **start_revision,
+                            const char **end_url,
+                            svn_opt_revision_t **end_revision,
+                            svn_ra_session_t *ra_session,
+                            const char *path,
+                            const svn_opt_revision_t *revision,
+                            const svn_opt_revision_t *start,
+                            const svn_opt_revision_t *end,
+                            svn_client_ctx_t *ctx,
+                            apr_pool_t *pool);
 
 
 /* Given PATH_OR_URL, which contains either a working copy path or an
@@ -169,14 +173,14 @@ svn_client__repos_locations (const char **start_url,
 
    Use POOL for all allocations. */
 svn_error_t *
-svn_client__ra_session_from_path (svn_ra_session_t **ra_session_p,
-                                  svn_revnum_t *rev_p,
-                                  const char **url_p,
-                                  const char *path_or_url,
-                                  const svn_opt_revision_t *peg_revision,
-                                  const svn_opt_revision_t *revision,
-                                  svn_client_ctx_t *ctx,
-                                  apr_pool_t *pool);
+svn_client__ra_session_from_path(svn_ra_session_t **ra_session_p,
+                                 svn_revnum_t *rev_p,
+                                 const char **url_p,
+                                 const char *path_or_url,
+                                 const svn_opt_revision_t *peg_revision,
+                                 const svn_opt_revision_t *revision,
+                                 svn_client_ctx_t *ctx,
+                                 apr_pool_t *pool);
 
 
 /* ---------------------------------------------------------------- */
@@ -194,6 +198,10 @@ typedef struct
   const char *base_dir;
   svn_wc_adm_access_t *base_access;
 
+  /* When true, makes sure temporary files are created
+     outside the working copy. */
+  svn_boolean_t read_only_wc;
+
   /* An array of svn_client_commit_item2_t * structures, present only
      during working copy commits. */
   apr_array_header_t *commit_items;
@@ -234,15 +242,15 @@ typedef struct
    NOTE: The reason for the _internal suffix of this function's name is to
    avoid confusion with the public API svn_client_open_ra_session(). */
 svn_error_t *
-svn_client__open_ra_session_internal (svn_ra_session_t **ra_session,
-                                      const char *base_url,
-                                      const char *base_dir,
-                                      svn_wc_adm_access_t *base_access,
-                                      apr_array_header_t *commit_items,
-                                      svn_boolean_t use_admin,
-                                      svn_boolean_t read_only_wc,
-                                      svn_client_ctx_t *ctx,
-                                      apr_pool_t *pool);
+svn_client__open_ra_session_internal(svn_ra_session_t **ra_session,
+                                     const char *base_url,
+                                     const char *base_dir,
+                                     svn_wc_adm_access_t *base_access,
+                                     apr_array_header_t *commit_items,
+                                     svn_boolean_t use_admin,
+                                     svn_boolean_t read_only_wc,
+                                     svn_client_ctx_t *ctx,
+                                     apr_pool_t *pool);
 
 
 
@@ -251,18 +259,17 @@ svn_client__open_ra_session_internal (svn_ra_session_t **ra_session,
 /*** Commit ***/
 
 /* Get the commit_baton to be used in couple with commit_callback. */
-svn_error_t *svn_client__commit_get_baton (void **baton,
-                                           svn_commit_info_t **info,
-                                           apr_pool_t *pool);
+svn_error_t *svn_client__commit_get_baton(void **baton,
+                                          svn_commit_info_t **info,
+                                          apr_pool_t *pool);
 
 /* The commit_callback function for storing svn_client_commit_info_t
    pointed by commit_baton. If the commit_info supplied by get_baton
    points to NULL after close_edit, it means the commit is a no-op.
 */
-svn_error_t *svn_client__commit_callback (svn_revnum_t revision,
-                                          const char *date,
-                                          const char *author,
-                                          void *baton);
+svn_error_t *svn_client__commit_callback(const svn_commit_info_t *commit_info,
+                                         void *baton,
+                                         apr_pool_t *pool);
 
 /* ---------------------------------------------------------------- */
 
@@ -273,9 +280,9 @@ svn_error_t *svn_client__commit_callback (svn_revnum_t revision,
    under PATH.  This is similar to checking the output of the status
    command.  CTX is used for the client's config options.  POOL is
    used for all temporary allocations. */
-svn_error_t * svn_client__can_delete (const char *path,
-                                      svn_client_ctx_t *ctx,
-                                      apr_pool_t *pool);
+svn_error_t * svn_client__can_delete(const char *path,
+                                     svn_client_ctx_t *ctx,
+                                     apr_pool_t *pool);
 
 
 /* ---------------------------------------------------------------- */
@@ -288,11 +295,11 @@ svn_error_t * svn_client__can_delete (const char *path,
    auto-props are disabled, set *PROPERTIES to NULL.
    Set *MIMETYPE to the mimetype, if any, or to NULL.
    Allocate the hash table, keys, values, and mimetype in POOL. */
-svn_error_t *svn_client__get_auto_props (apr_hash_t **properties,
-                                         const char **mimetype,
-                                         const char *path,
-                                         svn_client_ctx_t *ctx,
-                                         apr_pool_t *pool);
+svn_error_t *svn_client__get_auto_props(apr_hash_t **properties,
+                                        const char **mimetype,
+                                        const char *path,
+                                        svn_client_ctx_t *ctx,
+                                        apr_pool_t *pool);
                             
 
 /* The main logic for client deletion from a working copy. Deletes PATH
@@ -301,18 +308,24 @@ svn_error_t *svn_client__get_auto_props (apr_hash_t **properties,
    If DRY_RUN is TRUE all the checks are made to ensure that the delete can
    occur, but the working copy is not modifed.  If NOTIFY_FUNC is not
    null, it is called with NOTIFY_BATON for each file or directory deleted. */
-svn_error_t * svn_client__wc_delete (const char *path,
-                                     svn_wc_adm_access_t *adm_access,
-                                     svn_boolean_t force,
-                                     svn_boolean_t dry_run,
-                                     svn_wc_notify_func2_t notify_func,
-                                     void *notify_baton,
-                                     svn_client_ctx_t *ctx,
-                                     apr_pool_t *pool);
+svn_error_t * svn_client__wc_delete(const char *path,
+                                    svn_wc_adm_access_t *adm_access,
+                                    svn_boolean_t force,
+                                    svn_boolean_t dry_run,
+                                    svn_wc_notify_func2_t notify_func,
+                                    void *notify_baton,
+                                    svn_client_ctx_t *ctx,
+                                    apr_pool_t *pool);
+
+/* Return the list of WC paths to entries which would have been
+   deleted by an update/merge if not in "dry run" mode, or NULL if not
+   in "dry run" mode.  MERGE_CMD_BATON contains the list, which is
+   intended for direct modification. */
+apr_hash_t *svn_client__dry_run_deletions(void *merge_cmd_baton);
 
 /* ---------------------------------------------------------------- */
 
-/*** Checkout and update ***/
+/*** Checkout, update and switch ***/
 
 /* Update a working copy PATH to REVISION, and (if not NULL) set
    RESULT_REV to the update revision.  RECURSE if so commanded;
@@ -322,14 +335,14 @@ svn_error_t * svn_client__wc_delete (const char *path,
    will set *TIMESTAMP_SLEEP to TRUE if a sleep is required, and will
    not change *TIMESTAMP_SLEEP if no sleep is required. */
 svn_error_t *
-svn_client__update_internal (svn_revnum_t *result_rev,
-                             const char *path,
-                             const svn_opt_revision_t *revision,
-                             svn_boolean_t recurse,
-                             svn_boolean_t ignore_externals,
-                             svn_boolean_t *timestamp_sleep,
-                             svn_client_ctx_t *ctx,
-                             apr_pool_t *pool);
+svn_client__update_internal(svn_revnum_t *result_rev,
+                            const char *path,
+                            const svn_opt_revision_t *revision,
+                            svn_boolean_t recurse,
+                            svn_boolean_t ignore_externals,
+                            svn_boolean_t *timestamp_sleep,
+                            svn_client_ctx_t *ctx,
+                            apr_pool_t *pool);
 
 /* Checkout into PATH a working copy of URL at REVISION, and (if not
    NULL) set RESULT_REV to the checked out revision.  RECURSE if so
@@ -340,16 +353,33 @@ svn_client__update_internal (svn_revnum_t *result_rev,
    sleep is required, and will not change *TIMESTAMP_SLEEP if no sleep
    is required. */
 svn_error_t *
-svn_client__checkout_internal (svn_revnum_t *result_rev,
-                               const char *URL,
-                               const char *path,
-                               const svn_opt_revision_t *peg_revision,
-                               const svn_opt_revision_t *revision,
-                               svn_boolean_t recurse,
-                               svn_boolean_t ignore_externals,
-                               svn_boolean_t *timestamp_sleep,
-                               svn_client_ctx_t *ctx,
-                               apr_pool_t *pool);
+svn_client__checkout_internal(svn_revnum_t *result_rev,
+                              const char *URL,
+                              const char *path,
+                              const svn_opt_revision_t *peg_revision,
+                              const svn_opt_revision_t *revision,
+                              svn_boolean_t recurse,
+                              svn_boolean_t ignore_externals,
+                              svn_boolean_t *timestamp_sleep,
+                              svn_client_ctx_t *ctx,
+                              apr_pool_t *pool);
+
+/* Switch a working copy PATH to URL at REVISION, and (if not NULL)
+   set RESULT_REV to the switch revision.  RECURSE if so commanded.
+   If TIMESTAMP_SLEEP is NULL this function will sleep before
+   returning to ensure timestamp integrity.  If TIMESTAMP_SLEEP is not
+   NULL then the function will not sleep but will set *TIMESTAMP_SLEEP
+   to TRUE if a sleep is required, and will not change
+   *TIMESTAMP_SLEEP if no sleep is required. */
+svn_error_t *
+svn_client__switch_internal(svn_revnum_t *result_rev,
+                            const char *path,
+                            const char *url,
+                            const svn_opt_revision_t *revision,
+                            svn_boolean_t recurse,
+                            svn_boolean_t *timestamp_sleep,
+                            svn_client_ctx_t *ctx,
+                            apr_pool_t *pool);
 
 /* ---------------------------------------------------------------- */
 
@@ -384,23 +414,50 @@ svn_client__checkout_internal (svn_revnum_t *result_rev,
  * EDITOR/EDIT_BATON return the newly created editor and baton/
  */
 svn_error_t *
-svn_client__get_diff_editor (const char *target,
-                             svn_wc_adm_access_t *adm_access,
-                             const svn_wc_diff_callbacks2_t *diff_cmd,
-                             void *diff_cmd_baton,
-                             svn_boolean_t recurse,
-                             svn_boolean_t dry_run,
-                             svn_ra_session_t *ra_session, 
-                             svn_revnum_t revision,
-                             svn_wc_notify_func2_t notify_func,
-                             void *notify_baton,
-                             svn_cancel_func_t cancel_func,
-                             void *cancel_baton,
-                             const svn_delta_editor_t **editor,
-                             void **edit_baton,
-                             apr_pool_t *pool);
+svn_client__get_diff_editor(const char *target,
+                            svn_wc_adm_access_t *adm_access,
+                            const svn_wc_diff_callbacks2_t *diff_cmd,
+                            void *diff_cmd_baton,
+                            svn_boolean_t recurse,
+                            svn_boolean_t dry_run,
+                            svn_ra_session_t *ra_session, 
+                            svn_revnum_t revision,
+                            svn_wc_notify_func2_t notify_func,
+                            void *notify_baton,
+                            svn_cancel_func_t cancel_func,
+                            void *cancel_baton,
+                            const svn_delta_editor_t **editor,
+                            void **edit_baton,
+                            apr_pool_t *pool);
 
 
+/* ---------------------------------------------------------------- */
+
+/*** Editor for diff summary ***/
+
+/* Create an editor for a repository diff summary, i.e. comparing one
+ * repository version against the other and only providing information
+ * about the changed items without the text deltas.
+ *
+ * @a summarize_func is called with @a summarize_baton as parameter by the
+ * created svn_delta_editor_t for each changed item.
+ *
+ * See svn_client__get_diff_editor() for a description of the other
+ * parameters.
+ */
+svn_error_t *
+svn_client__get_diff_summarize_editor(const char *target,
+                                      svn_client_diff_summarize_func_t
+                                      summarize_func,
+                                      void *summarize_baton,
+                                      svn_ra_session_t *ra_session, 
+                                      svn_revnum_t revision,
+                                      svn_cancel_func_t cancel_func,
+                                      void *cancel_baton,
+                                      const svn_delta_editor_t **editor,
+                                      void **edit_baton,
+                                      apr_pool_t *pool);
+
 /* ---------------------------------------------------------------- */
 
 /*** Commit Stuff ***/
@@ -410,14 +467,14 @@ svn_client__get_diff_editor (const char *target,
 
    The day that 'svn switch' came into existence, our old commit
    crawler (svn_wc_crawl_local_mods) became obsolete.  It relied far
-   too heavily on the on-disk heirarchy of files and directories, and
+   too heavily on the on-disk hierarchy of files and directories, and
    simply had no way to support disjoint working copy trees or nest
    working copies.  The primary reason for this is that commit
    process, in order to guarantee atomicity, is a single drive of a
    commit editor which is based not on working copy paths, but on
    URLs.  With the completion of 'svn switch', it became all too
-   likely that the on-disk working copy heirarchy would no longer be
-   guaranteed to map to a similar in-repository heirarchy.
+   likely that the on-disk working copy hierarchy would no longer be
+   guaranteed to map to a similar in-repository hierarchy.
 
    Aside from this new brokenness of the old system, an unrelated
    feature request had cropped up -- the ability to know in advance of
@@ -501,14 +558,14 @@ svn_client__get_diff_editor (const char *target,
    CTX->CANCEL_BATON while harvesting to determine if the client has 
    cancelled the operation.  */
 svn_error_t *
-svn_client__harvest_committables (apr_hash_t **committables,
-                                  apr_hash_t **lock_tokens,
-                                  svn_wc_adm_access_t *parent_dir,
-                                  apr_array_header_t *targets,
-                                  svn_boolean_t nonrecursive,
-                                  svn_boolean_t just_locked,
-                                  svn_client_ctx_t *ctx,
-                                  apr_pool_t *pool);
+svn_client__harvest_committables(apr_hash_t **committables,
+                                 apr_hash_t **lock_tokens,
+                                 svn_wc_adm_access_t *parent_dir,
+                                 apr_array_header_t *targets,
+                                 svn_boolean_t nonrecursive,
+                                 svn_boolean_t just_locked,
+                                 svn_client_ctx_t *ctx,
+                                 apr_pool_t *pool);
 
 
 /* Recursively crawl the working copy path TARGET, harvesting
@@ -522,17 +579,17 @@ svn_client__harvest_committables (apr_hash_t **committables,
    CTX->CANCEL_BATON while harvesting to determine if the client has 
    cancelled the operation.  */
 svn_error_t *
-svn_client__get_copy_committables (apr_hash_t **committables,
-                                   const char *new_url,
-                                   const char *target,
-                                   svn_wc_adm_access_t *adm_access,
-                                   svn_client_ctx_t *ctx,
-                                   apr_pool_t *pool);
+svn_client__get_copy_committables(apr_hash_t **committables,
+                                  const char *new_url,
+                                  const char *target,
+                                  svn_wc_adm_access_t *adm_access,
+                                  svn_client_ctx_t *ctx,
+                                  apr_pool_t *pool);
                
 
 /* A qsort()-compatible sort routine for sorting an array of
    svn_client_commit_item_t's by their URL member. */
-int svn_client__sort_commit_item_urls (const void *a, const void *b);
+int svn_client__sort_commit_item_urls(const void *a, const void *b);
 
 
 /* Rewrite the COMMIT_ITEMS array to be sorted by URL.  Also, discover
@@ -542,9 +599,9 @@ int svn_client__sort_commit_item_urls (const void *a, const void *b);
    Afterwards, some of the items in COMMIT_ITEMS may contain data
    allocated in POOL. */
 svn_error_t *
-svn_client__condense_commit_items (const char **base_url,
-                                   apr_array_header_t *commit_items,
-                                   apr_pool_t *pool);
+svn_client__condense_commit_items(const char **base_url,
+                                  apr_array_header_t *commit_items,
+                                  apr_pool_t *pool);
 
 
 /* Commit the items in the COMMIT_ITEMS array using EDITOR/EDIT_BATON
@@ -561,17 +618,23 @@ svn_client__condense_commit_items (const char **base_url,
 
    If the caller wants to keep track of any outstanding temporary
    files left after the transmission of text and property mods,
-   *TEMPFILES is the place to look.  */
+   *TEMPFILES is the place to look.
+
+   MD5 checksums, if available,  for the new text bases of committed
+   files are stored in *DIGESTS, which maps const char* paths (from the
+   items' paths) to const unsigned char* digests.  DIGESTS may be
+   null.  */
 svn_error_t *
-svn_client__do_commit (const char *base_url,
-                       apr_array_header_t *commit_items,
-                       svn_wc_adm_access_t *adm_access,
-                       const svn_delta_editor_t *editor,
-                       void *edit_baton,
-                       const char *notify_path_prefix,
-                       apr_hash_t **tempfiles,
-                       svn_client_ctx_t *ctx,
-                       apr_pool_t *pool);
+svn_client__do_commit(const char *base_url,
+                      apr_array_header_t *commit_items,
+                      svn_wc_adm_access_t *adm_access,
+                      const svn_delta_editor_t *editor,
+                      void *edit_baton,
+                      const char *notify_path_prefix,
+                      apr_hash_t **tempfiles,
+                      apr_hash_t **digests,
+                      svn_client_ctx_t *ctx,
+                      apr_pool_t *pool);
 
 
 
@@ -602,11 +665,11 @@ svn_client__do_commit (const char *base_url,
 
    Use POOL for temporary allocation. */
 svn_error_t *
-svn_client__handle_externals (svn_wc_traversal_info_t *traversal_info,
-                              svn_boolean_t update_unchanged,
-                              svn_boolean_t *timestamp_sleep,
-                              svn_client_ctx_t *ctx,
-                              apr_pool_t *pool);
+svn_client__handle_externals(svn_wc_traversal_info_t *traversal_info,
+                             svn_boolean_t update_unchanged,
+                             svn_boolean_t *timestamp_sleep,
+                             svn_client_ctx_t *ctx,
+                             apr_pool_t *pool);
 
 
 /* Fetch externals definitions described by EXTERNALS, a hash of the
@@ -620,30 +683,30 @@ svn_client__handle_externals (svn_wc_traversal_info_t *traversal_info,
 
    Use POOL for temporary allocation. */
 svn_error_t *
-svn_client__fetch_externals (apr_hash_t *externals,
-                             svn_boolean_t is_export,
-                             svn_boolean_t *timestamp_sleep,
-                             svn_client_ctx_t *ctx,
-                             apr_pool_t *pool);
+svn_client__fetch_externals(apr_hash_t *externals,
+                            svn_boolean_t is_export,
+                            svn_boolean_t *timestamp_sleep,
+                            svn_client_ctx_t *ctx,
+                            apr_pool_t *pool);
 
 
 /* Perform status operations on each external in TRAVERSAL_INFO.  All
    other options are the same as those passed to svn_client_status(). */
 svn_error_t *
-svn_client__do_external_status (svn_wc_traversal_info_t *traversal_info,
-                                svn_wc_status_func2_t status_func,
-                                void *status_baton,
-                                svn_boolean_t get_all,
-                                svn_boolean_t update,
-                                svn_boolean_t no_ignore,
-                                svn_client_ctx_t *ctx,
-                                apr_pool_t *pool);
+svn_client__do_external_status(svn_wc_traversal_info_t *traversal_info,
+                               svn_wc_status_func2_t status_func,
+                               void *status_baton,
+                               svn_boolean_t get_all,
+                               svn_boolean_t update,
+                               svn_boolean_t no_ignore,
+                               svn_client_ctx_t *ctx,
+                               apr_pool_t *pool);
 
 
 
 /* Retrieves log message using *CTX->log_msg_func or
  * *CTX->log_msg_func2 callbacks.
- * Other argements same as svn_client_get_commit_log2_t */
+ * Other arguments same as svn_client_get_commit_log2_t. */
 svn_error_t * svn_client__get_log_msg(const char **log_msg,
                                       const char **tmp_file,
                                       const apr_array_header_t *commit_items,
