@@ -1448,7 +1448,11 @@ struct receiver_baton
 static void
 lock_receiver(void *userdata,
               const struct ne_lock *lock,
+#ifdef SVN_NEON_0_26
+              const ne_uri *uri,
+#else
               const char *uri,
+#endif
               const ne_status *status)
 {
   struct receiver_baton *rb = userdata;
@@ -1511,6 +1515,7 @@ svn_ra_dav__get_lock(svn_ra_session_t *session,
   struct receiver_baton *rb;
   svn_string_t fs_path;
   svn_error_t *err;
+  ne_uri uri;
 
   /* To begin, we convert the incoming path into an absolute fs-path. */
   url = svn_path_url_add_component(ras->url->data, path, pool);  
@@ -1533,6 +1538,14 @@ svn_ra_dav__get_lock(svn_ra_session_t *session,
 
   /* Ask neon to "discover" the lock (presumably by doing a PROPFIND
      for the DAV:supportedlock property). */
+  
+  /* ne_lock_discover wants just the path, so parse it out of the url. */
+  if (ne_uri_parse(url, &uri) == 0)
+    {
+      url = apr_pstrdup(pool, uri.path);
+      ne_uri_free(&uri);
+    }
+
   rv = ne_lock_discover(ras->sess, url, lock_receiver, rb);
 
   /* Did we get a <D:error> response? */
