@@ -2,7 +2,7 @@
  * lock.c: mod_dav_svn locking provider functions
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -139,7 +139,7 @@ unescape_xml(const char **output,
   if (apr_err)
     {
       char errbuf[1024];
-      (void *)apr_xml_parser_geterror(xml_parser, errbuf, sizeof(errbuf));
+      (void)apr_xml_parser_geterror(xml_parser, errbuf, sizeof(errbuf));
       return dav_new_error(pool, HTTP_INTERNAL_SERVER_ERROR,
                            DAV_ERR_LOCK_SAVE_LOCK, errbuf);
     }
@@ -173,7 +173,7 @@ dav_lock_to_svn_lock(svn_lock_t **slock,
                          DAV_ERR_LOCK_SAVE_LOCK,
                          "Only exclusive locks are supported.");
 
-  lock = svn_lock_create (pool);
+  lock = svn_lock_create(pool);
   lock->path = apr_pstrdup(pool, path);
   lock->token = apr_pstrdup(pool, dlock->locktoken->uuid_str);
 
@@ -499,11 +499,13 @@ dav_svn_get_locks(dav_lockdb *lockdb,
       return 0;
     }
 
-  /* The Big Lie: if the client ran 'svn lock --force', then we have
+  /* The Big Lie: if the client ran 'svn lock', then we have
      to pretend that there's no existing lock.  Otherwise mod_dav will
      throw '403 Locked' without even attempting to create a new
-     lock.  */
-  if (info->lock_steal)
+     lock.  For the --force case, this is required and for the non-force case,
+     we allow the filesystem to produce a better error for svn clients.
+  */
+  if (info->r->method_number == M_LOCK)
     {
       *locks = NULL;
       return 0;
@@ -537,8 +539,8 @@ dav_svn_get_locks(dav_lockdb *lockdb,
 
       /* Let svn clients know the creationdate of the slock. */
       apr_table_setn(info->r->headers_out, SVN_DAV_CREATIONDATE_HEADER,
-                     svn_time_to_cstring (slock->creation_date,
-                                          resource->pool));
+                     svn_time_to_cstring(slock->creation_date,
+                                         resource->pool));
       
       /* Let svn clients know who "owns" the slock. */
       apr_table_setn(info->r->headers_out, SVN_DAV_LOCK_OWNER_HEADER,
@@ -602,17 +604,17 @@ dav_svn_find_lock(dav_lockdb *lockdb,
     {
       /* Sanity check. */
       if (strcmp(locktoken->uuid_str, slock->token) != 0)
-        return dav_svn_convert_err(serr, HTTP_BAD_REQUEST,
-                               "Incoming token doesn't match existing lock.",
-                               resource->pool);
+        return dav_new_error(resource->pool, HTTP_BAD_REQUEST,
+                             DAV_ERR_LOCK_SAVE_LOCK,
+                             "Incoming token doesn't match existing lock.");
 
       svn_lock_to_dav_lock(&dlock, slock, FALSE,
                            resource->exists, resource->pool);
       
       /* Let svn clients know the creationdate of the slock. */
       apr_table_setn(info->r->headers_out, SVN_DAV_CREATIONDATE_HEADER,
-                     svn_time_to_cstring (slock->creation_date,
-                                          resource->pool));
+                     svn_time_to_cstring(slock->creation_date,
+                                         resource->pool));
 
       /* Let svn clients know the 'owner' of the slock. */
       apr_table_setn(info->r->headers_out, SVN_DAV_LOCK_OWNER_HEADER,
@@ -653,11 +655,13 @@ dav_svn_has_locks(dav_lockdb *lockdb,
       return 0;
     }
 
-  /* The Big Lie: if the client ran 'svn lock --force', then we have
+  /* The Big Lie: if the client ran 'svn lock', then we have
      to pretend that there's no existing lock.  Otherwise mod_dav will
      throw '403 Locked' without even attempting to create a new
-     lock.  */
-  if (info->lock_steal)
+     lock.  For the --force case, this is required and for the non-force case,
+     we allow the filesystem to produce a better error for svn clients.
+  */
+  if (info->r->method_number == M_LOCK)
     {
       *locks_present = 0;
       return 0;
@@ -767,7 +771,7 @@ dav_svn_append_locks(dav_lockdb *lockdb,
                                    "Could not begin a transaction", 
                                    resource->pool);
 
-      if ((serr = svn_fs_txn_root (&txn_root, txn, resource->pool)))
+      if ((serr = svn_fs_txn_root(&txn_root, txn, resource->pool)))
         return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
                                    "Could not begin a transaction", 
                                    resource->pool);
@@ -831,7 +835,7 @@ dav_svn_append_locks(dav_lockdb *lockdb,
      svn clients can fill in svn_lock_t->creation_date.  A generic DAV
      client should just ignore the header. */
   apr_table_setn(info->r->headers_out, SVN_DAV_CREATIONDATE_HEADER,
-                 svn_time_to_cstring (slock->creation_date, resource->pool));
+                 svn_time_to_cstring(slock->creation_date, resource->pool));
 
   /* A standard webdav LOCK response doesn't include any information
      about the owner of the lock.  ('DAV:owner' has nothing to do with
