@@ -1,20 +1,20 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
-# $HeadURL: http://svn.collab.net/repos/svn/branches/1.1.x/tools/hook-scripts/svnperms.py $
-# $LastChangedDate: 2003-09-09 19:35:09 -0700 (Tue, 09 Sep 2003) $
-# $LastChangedBy: dlr $
-# $LastChangedRevision: 7028 $
+# $HeadURL: http://svn.collab.net/repos/svn/branches/1.4.x/tools/hook-scripts/svnperms.py $
+# $LastChangedDate: 2005-09-23 18:19:04 +0000 (Fri, 23 Sep 2005) $
+# $LastChangedBy: niemeyer $
+# $LastChangedRevision: 16232 $
 
 import commands
 import sys, os
 import getopt
 import re
 
-__author__ = "Gustavo Niemeyer <niemeyer@conectiva.com>"
+__author__ = "Gustavo Niemeyer <gustavo@niemeyer.net>"
 
 class Error(Exception): pass
 
-SECTION = re.compile(r'\[([^]]+)\]')
+SECTION = re.compile(r'\[([^]]+?)(?:\s+extends\s+([^]]+))?\]')
 OPTION = re.compile(r'(\S+)\s*=\s*(.*)$')
 
 class Config:
@@ -43,8 +43,19 @@ class Config:
                 m = SECTION.match(line)
                 if m:
                     sectname = m.group(1)
-                    cursectdict = self._sections_dict.setdefault(sectname, {})
-                    cursectlist = []
+                    parentsectname = m.group(2)
+                    if parentsectname is None:
+                        # No parent section defined, so start a new section
+                        cursectdict = self._sections_dict.setdefault \
+                            (sectname, {})
+                        cursectlist = []
+                    else:
+                        # Copy the parent section into the new section
+                        parentsectdict = self._sections_dict.get \
+                            (parentsectname, {})
+                        cursectdict = self._sections_dict.setdefault \
+                            (sectname, parentsectdict.copy())
+                        cursectlist = self.walk(parentsectname)
                     self._sections_list.append((sectname, cursectlist))
                     optname = None
                 elif cursectdict is None:
