@@ -11,12 +11,13 @@ FS_TYPE="$3"
 # Compute local vars
 LOG_FILE="$LOG_FILE_PREFIX.$BUILD_TYPE.$RA_TYPE.$FS_TYPE"
 TEST="`$GUESS` $BUILD_TYPE $RA_TYPE $FS_TYPE"
-REV="`$SVN st -v $SVN_REPO/README | $CUT -c 12-17 | $SED -e 's/^ *//'`"
+REV="`$SVN st -v $SVN_SOURCE/README | $CUT -c 12-17 | $SED -e 's/^ *//'`"
 
 # Prime and initialize the log file
 $CP_F "$LOG_FILE_PREFIX.$BUILD_TYPE" $LOG_FILE
 echo >> $LOG_FILE
 echo "TEST: $REVPREFIX$REV on $TEST" >> $LOG_FILE
+echo "TIME: $($DATE '+%Y-%m-%d %H:%M:%S %z')" >> $LOG_FILE
 echo >> $LOG_FILE
 
 # Check the build type
@@ -58,38 +59,29 @@ PASS
 START "check object directory" "Checking object directory..."
 test -d $TEST_ROOT/$OBJ || FAIL; PASS
 START "check svn executable" "Checking svn executable..."
-test -x $TEST_ROOT/$OBJ/subversion/clients/cmdline/svn || FAIL; PASS
+test -x $INST_DIR/$SVN_NAME/bin/svn || FAIL; PASS
 START "check svnadmin executable" "Checking svnadmin executable..."
-test -x $TEST_ROOT/$OBJ/subversion/svnadmin/svnadmin || FAIL; PASS
+test -x $INST_DIR/$SVN_NAME/bin/svnadmin || FAIL; PASS
 START "check svnlook executable" "Checking svnlook executable..."
-test -x $TEST_ROOT/$OBJ/subversion/svnlook/svnlook || FAIL; PASS
+test -x $INST_DIR/$SVN_NAME/bin/svnlook || FAIL; PASS
 START "check svnserve executable" "Checking svnserve executable..."
-test -x $TEST_ROOT/$OBJ/subversion/svnserve/svnserve || FAIL; PASS
+test -x $INST_DIR/$SVN_NAME/bin/svnserve || FAIL; PASS
 START "check svnversion executable" "Checking svnversion executable..."
-test -x $TEST_ROOT/$OBJ/subversion/svnversion/svnversion || FAIL; PASS
+test -x $INST_DIR/$SVN_NAME/bin/svnversion || FAIL; PASS
 
 # Build has initially mounted ramdisk for us, but this
 # script will at the end to do unmount, so check if it is mounted or not
 # and if it is not, do initial fire up for it
 if test "xyes" = "x$RAMDISK";
 then
-    test -x $TEST_ROOT/$OBJ/subversion/tests/clients || {
-        START "re-initializing ramdisk" "Re-initializing ramdisk"
-        mount_ramdisk "$TEST_ROOT/$OBJ/subversion/tests" \
-            >> "$LOG_FILE" 2>&1 || FAIL
-        cd "$TEST_ROOT/$OBJ"
-        $MAKE  mkdir-init > "$LOG_FILE.ramdisk" 2>&1
-        test $? = 0 || {
-            FAIL_LOG "$LOG_FILE.ramdisk"
-            FAIL
-        }
-        $MAKE $MAKE_OPTS > "$LOG_FILE.ramdisk" 2>&1
-        test $? = 0 || {
-            FAIL_LOG "$LOG_FILE.ramdisk"
-            FAIL
-        }
-        PASS
-    }
+    reinitialize_ramdisk
+fi
+
+if test "xyes" = "x$INTERMEDIATE_CLEANUP";
+then
+    # Flag the tests to cleanup after themselves to avoid requiring
+    # hundreds of MBs of storage at once for test data.
+    CHECK_ARGS="$CHECK_ARGS CLEANUP=1"
 fi
 
 # Prepare the server
@@ -100,7 +92,8 @@ case $CHECK_TARGET in
     svncheck)
         START "run svnserve" "Running svnserve..."
         $TEST_ROOT/$OBJ/subversion/svnserve/svnserve -d \
-            -r $TEST_ROOT/$OBJ/subversion/tests/clients/cmdline \
+            --listen-port $SVNSERVE_PORT \
+            -r $TEST_ROOT/$OBJ/$RA_SVN_REPO_ROOT \
             >> $LOG_FILE 2>&1
         test $? = 0 || FAIL
         PASS
@@ -111,13 +104,14 @@ case $CHECK_TARGET in
                        | $SED -e 's/^ *//' | $CUT -f 1 -d ' ' -s`"
         test -n "$SVNSERVE_PID" || FAIL
         PASS
+        CHECK_ARGS="$CHECK_ARGS $RA_SVN_CHECK_ARGS"
         ;;
     davcheck)
         START "run $HTTPD_NAME" "Running $HTTPD_NAME..."
         $CP_F "$TEST_ROOT/$HTTPD_NAME.conf" \
             "$INST_DIR/$HTTPD_NAME/conf/httpd.conf" || FAIL
 
-        $CP_F "$TEST_ROOT/mod_dav_svn_$BUILD_TYPE.conf" \
+        $CP_F "$TEST_ROOT/mod_dav_${SVN_NAME}.conf" \
             "$INST_DIR/$HTTPD_NAME/conf/mod_dav_svn.conf" || FAIL
 
         "$INST_DIR/$HTTPD_NAME/bin/apachectl" start \
@@ -154,13 +148,16 @@ kill_svnserve() {
 ts_start=`$DATE +"%s"`
 
 START "make $CHECK_TARGET" "Testing $RA_TYPE on $FS_TYPE..."
-CHECK_LOG_FILE="$TEST_ROOT/LOG_svn_check_${BUILD_TYPE}_${RA_TYPE}_${FS_TYPE}"
+CHECK_LOG_FILE="$LOG_FILE_DIR/LOG_svn_check_${BUILD_TYPE}_${RA_TYPE}_${FS_TYPE}"
 cd $TEST_ROOT/$OBJ
 if test "$CHECK_TARGET" = "davcheck";
 then
     # At the moment we can't give repository url with
     # make davcheck, so use check & BASE_URL here for the present
     $MAKE check $CHECK_ARGS > $CHECK_LOG_FILE 2>&1
+elif test "$CHECK_TARGET" = "svncheck";
+then
+    $MAKE check $CHECK_ARGS > $CHECK_LOG_FILE 2>&1
 else
     $MAKE $CHECK_TARGET $CHECK_ARGS > $CHECK_LOG_FILE 2>&1
 fi
@@ -172,7 +169,7 @@ test $? = 0 || {
 
     # Prepare the log file for the mailer
     $GZIP_C < "tests.log" \
-            > "$TEST_ROOT/tests.$BUILD_TYPE.$RA_TYPE.$FS_TYPE.log.gz"
+            > "$LOG_FILE_DIR/tests.$BUILD_TYPE.$RA_TYPE.$FS_TYPE.log.gz"
     FAIL kill_svnserve
 }
 PASS
@@ -183,3 +180,5 @@ START "Timer: make $CHECK_TARGET $(($ts_stop - $ts_start)) sec" \
 PASS
 
 kill_svnserve
+echo "TIME: $($DATE '+%Y-%m-%d %H:%M:%S %z')" >> $LOG_FILE
+
